import pysmt
from pysmt.shortcuts import Symbol
from pysmt.smtlib.parser import PysmtSyntaxError, SmtLibParser


def make_pysmt_const_from_type(val, type):
    return getattr(pysmt.shortcuts, str(type))(val)


class SMTParser:
    def __init__(self, tokens):
        self.p = SmtLibParser()
        self.tokens = tokens

    def expect(self, *allowed):
        t = self.tokens.consume()
        if t not in allowed:
            raise PysmtSyntaxError(f"Invalid token, expected any of {allowed}, got '{t}'")
        return t

    def expect_assignment_tuple(self):
        self.expect("(")
        cmd = self.expect("define-fun")
        vname = self.p.parse_atom(self.tokens, cmd)
        self.expect("(")
        self.expect(")")
        t = self.p.parse_type(self.tokens, cmd)
        value = self.p.get_expression(self.tokens)
        self.expect(")")

        return Symbol(vname, t), getattr(pysmt.shortcuts, t.name)(value.constant_value())

    def consume_assignment_list(self):
        self.expect("(")
        self.expect("model")
        """Parses a list of expressions from the tokens"""

        assignments = []
        while True:
            next_token = self.tokens.consume()
            self.tokens.add_extra_token(next_token)  # push it back
            if next_token == ")":
                break

            assignments.append(self.expect_assignment_tuple())

        self.expect(")")

        return assignments



import unittest

import common_backend_smt_solver

import claripy


class SmtLibSolverTest_Z3(common_backend_smt_solver.SmtLibSolverTestBase):
    @unittest.skip("Skip these test for now because of a problem with pysmt")
    def get_solver(self):
        solver = claripy.SolverPortfolio(
            solvers=[
                claripy.SolverComposite(
                    template_solver_string=claripy.SolverCompositeChild(
                        backend=claripy.backend_manager.backends.smtlib_cvc4
                    )
                ),
                claripy.SolverComposite(
                    template_solver_string=claripy.SolverCompositeChild(
                        backend=claripy.backend_manager.backends.smtlib_z3
                    )
                ),
                claripy.SolverComposite(
                    template_solver_string=claripy.SolverCompositeChild(
                        backend=claripy.backend_manager.backends.smtlib_z3str
                    )
                ),
            ]
        )
        return solver


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(SmtLibSolverTest_Z3)
    unittest.TextTestRunner(verbosity=2).run(suite)



from typing import TYPE_CHECKING, Union

from claripy import BoolV

if TYPE_CHECKING:
    from claripy.ast.bool import Bool


class ConstraintFixerMixin:
    def add(self, constraints: Union["Bool", list["Bool"], set["Bool"], tuple["Bool", ...]], **kwargs) -> list["Bool"]:
        constraints = [constraints] if not isinstance(constraints, list | tuple | set) else constraints

        if len(constraints) == 0:
            return []

        constraints = [BoolV(c) if isinstance(c, bool) else c for c in constraints]
        return super().add(constraints, **kwargs)
